@@ -44,7 +44,9 @@
 #include <sidplay/fformat.h>
 #include <sidplay/myendian.h>
 
-#define PROGRAM_VERSION "0.3.7"
+#include <vector>
+
+#define PROGRAM_VERSION "0.3.9"
 #define PROGRAM_NAME "msid - " PROGRAM_VERSION
 
 #define APP_WINDOW_HEIGHT 400 // ~hildonwindow workspace
@@ -59,6 +61,7 @@ static GtkWidget *status_text=NULL;
 extern GtkWidget *player_song_list;
 extern GtkWidget *loader_song_list;
 extern GtkWidget *player_play_button;
+extern GtkWidget *player_pause_button;
 extern GtkWidget *player_read_button;
 extern GtkWidget *player_del_button;
 extern GtkWidget *subsong_selector;
@@ -67,6 +70,7 @@ extern GtkWidget *config_audio_plugin_button;
 extern GtkWidget *config_search_plugin_button;
 extern GtkWidget *config_audio_plugin_combo;
 extern GtkWidget *config_search_plugin_combo;
+extern GtkWidget *output_mode_toggle;
 
 extern GtkWidget *hvsc_path_selection_button;
 extern GtkWidget *hvsc_path_label;
@@ -87,6 +91,9 @@ static int SUBSONG = 1;
 
 char download_path[256];
 
+static GtkTreePath *current_selection = NULL;
+
+
 /* these settings are used by default *only* if everything else fails */
 #define DEFAULT_AUDIO_PLUGIN "libmsid_pulse.so"
 #define DEFAULT_SEARCH_PLUGIN "libmsid_c64org.so"
@@ -115,6 +122,8 @@ enum
 
 G_LOCK_DEFINE_STATIC(msid_state);
 static volatile unsigned int msid_state = READY;
+G_LOCK_DEFINE_STATIC(msid_pause);
+static volatile unsigned int msid_pause = 0;
 
 
 typedef struct _msid_plugin_iface
@@ -136,15 +145,38 @@ init_msid_machine (msid_machine *m);
 void
 update_status_text (const char *text, const char *color);
 
+static void
+select_song (GtkTreeView *treeview, gpointer data);
 
+static void
+modify_subsong_selector();
 
 void *player_thread(void *arg)
 {
   unsigned int ready = 0;
   msid_machine *m = (msid_machine*) arg;
-  ubyte* buffer;
+  ubyte* buffer = 0;
   int duration = 0;
 
+#ifdef HILDON
+#ifndef __arm__
+  fprintf(stderr, "playback not supported in scratchbox\n");
+    G_LOCK(msid_state);
+    msid_state = READY;
+    G_UNLOCK(msid_state);
+    return NULL;
+#endif
+#endif
+  
+  // final sanity check
+  if (!m->song) {
+    G_LOCK(msid_state);
+    msid_state = READY;
+    G_UNLOCK(msid_state);
+    fprintf(stderr, "critical - no song?!1\n");
+    return NULL;
+  }
+
   init_msid_machine (m); /* init driver */
 
   buffer = new ubyte[m->driver->bsize()];
@@ -159,15 +191,27 @@ void *player_thread(void *arg)
     if (msid_state == STOP) {
       ready = 1;
     }
+
     G_UNLOCK(msid_state);
 
-    sidEmuFillBuffer (m->engine, tune, buffer, m->driver->bsize());
-    m -> driver -> play_stream (buffer, m->driver->bsize());
+    G_LOCK(msid_pause);
+    G_UNLOCK(msid_pause);
+
+    sidEmuFillBuffer(m->engine, tune, buffer, m->driver->bsize());
+
+    // stop playback in case of error
+    if (!m->driver->play_stream(buffer, m->driver->bsize())) {
+      ready = 1;
+    }
+    
+    // TODO - debug byteCount ...
   }
 
   delete [] buffer;
 
-  m -> driver -> close();
+  m->driver->close();
+
+ exit_player_thread :
 
   G_LOCK(msid_state);
   msid_state = READY;
@@ -179,7 +223,6 @@ void *player_thread(void *arg)
 
 
 
-
 /*
  * set fullscreen state for window
  */
@@ -308,14 +351,20 @@ static void
 init_msid_machine (msid_machine *m)
 {
   struct emuConfig cfg;
+  int channels = 1;
 
-  m -> driver = (msid_driver *) audio_iface.plugin_object;
+  // stereo output?
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(output_mode_toggle)) == TRUE) {
+    channels++;
+  }
+  
+  m->driver = (msid_driver *) audio_iface.plugin_object;
 
-  m -> engine.getConfig (cfg);
-  m -> driver -> set_config (&cfg);
-  m -> engine.setConfig (cfg);
+  m->driver->initialize(NULL, 0, channels);
 
-  m -> driver->initialize (NULL, 0, 0);
+  m->engine.getConfig(cfg);
+  m->driver->set_config(&cfg);
+  m->engine.setConfig(cfg);
 }
 
 static void
@@ -330,25 +379,25 @@ load_screenshot (GtkWidget *widget,
   gchar *filepath=NULL;
   GError *error = NULL;
 
-  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(player_song_list));
-  if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
-    path = gtk_tree_model_get_path(model, &iter);
-    if(!path) {
-      return;
-    }
-    column = gtk_tree_view_get_column(GTK_TREE_VIEW(player_song_list), 0);
-    gtk_tree_selection_get_selected(selection, &model, &iter);
-    gtk_tree_model_get(model, &iter, PLAYER_LIST_SONG_PATH, &filepath, -1);
-    gtk_tree_path_free(path);
-
-    if (!filepath) {
-      return;
-    }
-
-    gtk_widget_set_sensitive (widget, FALSE);
-
-    set_game_screenshot (filepath);
+  if (!current_selection) {
+    return;
   }
+
+  model = gtk_tree_view_get_model(GTK_TREE_VIEW(player_song_list));
+
+  if (gtk_tree_model_get_iter (model, &iter, current_selection)) {
+    gtk_tree_model_get(model, &iter, PLAYER_LIST_SONG_PATH, &filepath, -1);
+  }
+
+  if (!filepath) {
+    return;
+  }
+
+  update_status_text ("LOADING IMAGES", C64_FG);
+
+  gtk_widget_set_sensitive (widget, FALSE);
+  set_game_screenshot (filepath);
+
 }
 
 static void
@@ -461,85 +510,26 @@ load_search_plugin (GtkWidget *widget,
 
 
 static void
-play_stop_song (GtkWidget *widget,
-                gpointer data)
+pause_song (GtkWidget *widget,
+	    gpointer data)
 {
-  GtkTreeSelection *selection;
-  GtkTreeModel     *model;
-  GtkTreeIter       iter;
-  GtkTreePath *path;
-  GtkTreeViewColumn *column;
-  gchar *filepath=NULL;
-  GError *error = NULL;
-
-  msid_machine *m;
-
-  m = (msid_machine*) data;
-
-  // switch play|stop button label
-  G_LOCK(msid_state);
-  switch (msid_state)
-  {
-  case PLAY :
-    msid_state = STOP;
-    G_UNLOCK(msid_state);
-    update_status_text ("READY.", "#aaaaff");
-    gtk_button_set_label(GTK_BUTTON(player_play_button), "PLAY");
-    G_UNLOCK(msid_state);
-    return;
-    break;
-  case READY :
-    msid_state = PLAY;
-    gtk_button_set_label(GTK_BUTTON(player_play_button), "STOP");
-    break;
-  default :
-    G_UNLOCK(msid_state);
-    return;
-    break;
-  }
-  G_UNLOCK(msid_state);
-
-  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(player_song_list));
-  if (gtk_tree_selection_get_selected(selection, &model, &iter))
-  {
-    path = gtk_tree_model_get_path(model, &iter);
-    if(!path)
-    {
-      return;
-    }
-
-    column = gtk_tree_view_get_column(GTK_TREE_VIEW(player_song_list), 0);
-    gtk_tree_selection_get_selected(selection, &model, &iter);
-    gtk_tree_model_get(model, &iter, PLAYER_LIST_SONG_PATH, &filepath, -1);
-    gtk_tree_path_free(path);
-
-    if(!filepath)
-    {
-      return;
-    }
-    else
-    {
-      m -> song = filepath;
-      m -> song_no = SUBSONG;
-
-      g_thread_create(player_thread, (void*) data, FALSE, &error);
-      return;
-    }
-  }
+  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(player_pause_button)) ? G_LOCK(msid_pause) : G_UNLOCK(msid_pause);
 }
 
+
 /* FIXME */
 static unsigned int
 stop_sid_machine()
 {
-  unsigned int ready = 0, timer = 10, result = 1;
+  unsigned int ready = 0, timer = 100, result = 1;
+
   G_LOCK (msid_state);
   msid_state = STOP;
   G_UNLOCK(msid_state);
 
   while (!ready)
   {
-    usleep (500);
+    usleep (10000);
 
     G_LOCK(msid_state);
     if (msid_state == READY)
@@ -561,6 +551,107 @@ stop_sid_machine()
 }
 
 
+
+static void
+play_stop_song (GtkWidget *widget,
+                gpointer data)
+{
+  GtkTreeSelection *selection;
+  GtkTreeModel     *model;
+  GtkTreeIter       iter;
+  GtkTreePath *path;
+  GtkTreeViewColumn *column;
+  gchar *filepath=NULL;
+  GError *error = NULL;
+
+  msid_machine *m = (msid_machine*) data;
+
+  if (!current_selection) {
+    update_status_text ("NO SONG SELECTED", "#ffaaaa");
+    return;
+  }
+
+  // switch play|stop button label
+  G_LOCK(msid_state);
+
+  switch (msid_state)
+  {
+  case PLAY :
+
+    G_UNLOCK(msid_state);
+
+    stop_sid_machine();
+
+    update_status_text ("READY.", "#aaaaff");
+    gtk_widget_set_sensitive(player_play_button, FALSE);
+    gtk_widget_set_sensitive(player_pause_button, FALSE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(player_pause_button), FALSE);
+    G_UNLOCK(msid_pause);
+    return;
+
+    break;
+
+  case READY :
+
+    msid_state = PLAY;
+    gtk_widget_set_sensitive(player_play_button, TRUE);
+    gtk_widget_set_sensitive(player_pause_button, TRUE);
+    break;
+
+  default :
+    fprintf(stderr, "warning default path hit in switch : %s\n", __func__);
+    G_UNLOCK(msid_state);
+    return;
+
+    break;
+  }
+
+  G_UNLOCK(msid_state);
+
+  g_thread_create(player_thread, (void*) data, FALSE, &error);
+
+  return;
+
+
+  /*
+
+  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(player_song_list));
+  if (gtk_tree_selection_get_selected(selection, &model, &iter))
+  {
+    path = gtk_tree_model_get_path(model, &iter);
+    if(!path)
+    {
+      fprintf(stderr, "no path\n");
+      return;
+    }
+
+    column = gtk_tree_view_get_column(GTK_TREE_VIEW(player_song_list), 0);
+    gtk_tree_selection_get_selected(selection, &model, &iter);
+    gtk_tree_model_get(model, &iter, PLAYER_LIST_SONG_PATH, &filepath, -1);
+    gtk_tree_path_free(path);
+
+    if(!filepath)
+    {
+      return;
+    }
+    else
+    {
+      m->song = filepath;
+      m->song_no = SUBSONG;
+
+      g_thread_create(player_thread, (void*) data, FALSE, &error);
+      return;
+    }
+  } else {
+    fprintf(stderr, "no selection\n");
+  }
+
+  */
+
+
+}
+
+
 static void
 quit_msid (GtkMenuItem *menuitem, gpointer data)
 {
@@ -578,43 +669,101 @@ play_selection (GtkTreeView       *tree_view,
 {
   unsigned int ready = 0, timer = 10;
 
+  GtkTreeModel     *model;
+  GtkTreeIter       iter;
+  gchar *filepath=NULL;
+  GError *error = NULL;
+
   G_LOCK (msid_state);
   switch (msid_state)
   {
   case PLAY :
     /* stop machine first and then play */
-    msid_state = STOP;
     G_UNLOCK(msid_state);
-
-    stop_sid_machine (); // blocking func
-
-    play_stop_song (NULL, user_data);
-    break;
+    stop_sid_machine ();
 
   default :
     G_UNLOCK(msid_state);
-    play_stop_song (NULL, user_data);
+
+    {
+      msid_machine *m = (msid_machine*) user_data;
+
+      model = gtk_tree_view_get_model(tree_view);
+
+      // ATTENTION - selection changes here
+      // this all hacking here is because of HILDON pannablearea sucks
+      // selection possibility from gtktreeview .. this breaks proper native gtk support
+      // now as well :/
+
+      if (current_selection) {
+	gtk_tree_path_free(current_selection);
+      }
+      current_selection = gtk_tree_path_copy(path);
+
+      // update subsong selector only when selection made,
+      // not on cursor change
+      modify_subsong_selector();
+
+      select_song (tree_view, NULL);
+
+      if (gtk_tree_model_get_iter (model, &iter, path)) {
+
+	gtk_tree_model_get(model, &iter, PLAYER_LIST_SONG_PATH, &filepath, -1);
+
+	if(!filepath) {
+	  return;
+	} else {
+
+	  if (m->song) {
+	    g_free(m->song);
+	  }
+
+	  m->song = filepath;
+	  m->song_no = SUBSONG;
+
+	  play_stop_song (NULL, user_data);
+	  return;
+	}
+
+      }
+    }
+
     break;
   }
 }
 
+static void
+handle_selection (GtkTreeView       *tree_view,
+		  GtkTreePath       *path,
+		  GtkTreeViewColumn *column,
+		  gpointer           user_data)
+{
+  GtkTreeModel     *model;
+
+  model = gtk_tree_view_get_model(tree_view);
+
+  play_selection(tree_view, path, column, user_data);
+}
+
 
 static unsigned int
 get_subsong_amount()
 {
-  GtkTreeSelection *selection;
   GtkTreeModel     *model;
   GtkTreeIter       iter;
-  unsigned int amount;
+  unsigned int amount = 0;
+
+  if (!current_selection) {
+    return 0;
+  }
 
   model = gtk_tree_view_get_model(GTK_TREE_VIEW(player_song_list));
-  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(player_song_list));
 
-  if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
+  if (gtk_tree_model_get_iter (model, &iter, current_selection)) {
     gtk_tree_model_get (model, &iter, PLAYER_LIST_SONG_AMOUNT, &amount, -1);
-    return amount;
   }
-  return 0;
+
+  return amount;
 }
 
 static void
@@ -625,21 +774,41 @@ subsong_changed (GtkComboBox *widget, gpointer data)
   GtkTreeIter       iter;
   unsigned int amount;
 
+  msid_machine *m = (msid_machine*) data;
+
   amount = get_subsong_amount();
 
   SUBSONG = gtk_combo_box_get_active (widget);
-  if (SUBSONG < 0 || SUBSONG > amount)
-    SUBSONG=1;
+  if (SUBSONG < 0 || SUBSONG > amount) {
+    SUBSONG = 1;
+  }
 
   if (ignore_subsong_change) {
     ignore_subsong_change = FALSE;
     return;
   }
 
+  G_LOCK(msid_state);
   if (msid_state != READY) {
-    stop_sid_machine ();
-    play_stop_song (NULL, data);
+
+    G_UNLOCK(msid_state);
+
+    stop_sid_machine();
+
+    G_LOCK(msid_state);
+    if (msid_state != READY) {
+      fprintf(stderr, "stop did not work ok :/\n");
+    }
+    G_UNLOCK(msid_state);
+
+    m->song_no = SUBSONG;
+
+    play_stop_song(NULL, data);
+
+    return;
   }
+  G_UNLOCK(msid_state);
+
 }
 
 
@@ -705,18 +874,26 @@ delete_current_selection (void)
   char buffer[256];
 
   GtkWidget *dialog;
+  GtkWidget *label;
 
-  model     = gtk_tree_view_get_model (GTK_TREE_VIEW(player_song_list));
-  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(player_song_list));
+  if (!current_selection) {
+    return;
+  }
 
-  if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
-    GtkWidget *label;
-    gtk_tree_model_get (model, &iter, PLAYER_LIST_SONG_PATH, &filepath, -1);
-    gtk_tree_model_get (model, &iter, PLAYER_LIST_SONG_NAME, &name, -1);
+  model = gtk_tree_view_get_model (GTK_TREE_VIEW(player_song_list));
 
-    snprintf (buffer, 256, "Delete '%s'?", name);
+  if (gtk_tree_model_get_iter (model, &iter, current_selection)) {
+    gtk_tree_model_get(model, &iter, PLAYER_LIST_SONG_PATH, &filepath, -1);
+    gtk_tree_model_get(model, &iter, PLAYER_LIST_SONG_NAME, &name, -1);
+  }
 
-    label = gtk_label_new (buffer);
+  if (!filepath || !name) {
+    return;
+  }
+
+  snprintf(buffer, 256, "Delete '%s'?", name);
+
+  label = gtk_label_new(buffer);
 
 #ifdef HILDON
     dialog = hildon_note_new_confirmation (GTK_WINDOW(app_window),
@@ -740,21 +917,18 @@ delete_current_selection (void)
     {
     case GTK_RESPONSE_ACCEPT:
     case GTK_RESPONSE_OK:
-      path = gtk_tree_model_get_path (model, &iter);
 
       // if there is previous slot
-      if (gtk_tree_path_prev (path))
+      if (gtk_tree_path_prev (current_selection))
 	gtk_tree_view_set_cursor (GTK_TREE_VIEW (player_song_list),
-				  path, NULL, FALSE);
+				  current_selection, NULL, FALSE);
       else {
-	gtk_tree_path_next (path);
-	if (gtk_tree_model_get_iter (model, &next, path))
+	gtk_tree_path_next (current_selection);
+	if (gtk_tree_model_get_iter (model, &next, current_selection))
 	  gtk_tree_view_set_cursor (GTK_TREE_VIEW (player_song_list),
-				    path, NULL, FALSE);
+				    current_selection, NULL, FALSE);
       }
 
-      gtk_tree_path_free (path);
-
       gtk_list_store_remove (GTK_LIST_STORE(model), &iter);
       unlink (filepath);
 
@@ -766,9 +940,9 @@ delete_current_selection (void)
     default :
       break;
     }
-  }
 }
 
+
 static void
 delete_song (GtkWidget *widget,
 	     gpointer data)
@@ -799,9 +973,8 @@ static void
 select_song (GtkTreeView *treeview,
              gpointer data)
 {
-  modify_subsong_selector ();
 
-  /* since song changed - we can release image_button for the user again */
+  /* since cursor changed - we can release image_button for the user again */
   gtk_widget_set_sensitive (image_button, TRUE);
 
   /* user cannot control animation anymore - song has changed! */
@@ -840,8 +1013,8 @@ static void
 show_from_panel (GtkStatusIcon *status_icon,
 		 gpointer       user_data)
 {
-  /* no - do not let user remove icon if it's still blinking,
-     see timeout function below for why not */
+  /* no - do not let user remove icon if it's still blinking,  
+   see timeout function below for why not */
   if (gtk_status_icon_get_blinking (status_icon))
     return;
   gtk_widget_show (app_window);
@@ -992,7 +1165,7 @@ show_help_dialog (GtkMenuItem *menuitem, gpointer data)
   GtkWidget *info;
   GtkWidget *hbox;
 
-  char txt[] = "Use <span foreground=\"#aaaaff\">Search</span> tab to download sid files, when doubletapped they will appear on <span foreground=\"#aaaaff\">Player</span> tab. When a song is selected in player tab, use <span foreground=\"#aaaaff\">Screenshot</span> tab to display screenshots related to the song, works with most game sid tunes.\n\nEnjoy!";
+  char txt[] = "Use <span foreground=\"#aaaaff\">Search</span> tab to download sid files, when tapped they will appear on <span foreground=\"#aaaaff\">Player</span> tab. When a song is selected in player tab, use <span foreground=\"#aaaaff\">Screenshot</span> tab to display screenshots related to the song, works with most game sid tunes.\n\nEnjoy!";
 
   GtkWidget *dialog = gtk_dialog_new_with_buttons ("Help!",
 						   GTK_WINDOW(app_window),
@@ -1106,13 +1279,22 @@ class "GtkWindow" style "window"
   gtk_rc_parse_string (rc_buffer);
 }
 
+static void output_mode_changed (GtkToggleButton *togglebutton,
+				 gpointer         user_data)
+{
+  // HACK - restart of the engine needed currently
+  // TODO - figure out nice way without complete restart
+
+  update_status_text ("OUTPUT MODE CHANGED", C64_FG);
+
+  subsong_changed (NULL, user_data);
+}
+
 
 static void
 show_window (gpointer *window)
 {
   printf("show window %p!\n", window);
-
-
   gtk_widget_show (GTK_WIDGET(window));
 }
 
@@ -1139,11 +1321,10 @@ int main (int argc, char **argv)
     *loader_label,
     *image_label,
 
-    /*
-    *player_button,
-    *loader_button,
-    *image_button,
-    */
+
+    *stack_player_button,
+    *stack_loader_button,
+    *stack_image_button,
 
     *player_page,
     *loader_page,
@@ -1165,6 +1346,12 @@ int main (int argc, char **argv)
   msid_driver *driver;
   msid_search_plugin *splugin;
 
+  msid_state = READY;
+
+  //  memset(&machine, 0, sizeof(msid_machine));
+
+  machine.song = 0;
+
   g_thread_init(NULL);
   gdk_threads_init();
   gdk_threads_enter();
@@ -1333,9 +1520,8 @@ int main (int argc, char **argv)
 
 
   /* force notebook for arm device */
-  //#ifdef HILDON
 #define NOTEBOOK
-  //#endif
+
 
 #ifdef NOTEBOOK
   mother_container = gtk_notebook_new();
@@ -1371,13 +1557,16 @@ int main (int argc, char **argv)
   loader_window = hildon_stackable_window_new();
   image_window  = hildon_stackable_window_new();
 
+
   gtk_container_add (GTK_CONTAINER(player_window), player_page);
   gtk_container_add (GTK_CONTAINER(loader_window), loader_page);
   gtk_container_add (GTK_CONTAINER(image_window), image_page);
 
+  /*
   hildon_program_add_window (program, HILDON_WINDOW (player_window));
   hildon_program_add_window (program, HILDON_WINDOW (loader_window));
   hildon_program_add_window (program, HILDON_WINDOW (image_window));
+  */
 
   printf("player window pointer %p!\n", player_window);
 
@@ -1410,7 +1599,11 @@ int main (int argc, char **argv)
   item1 = gtk_menu_item_new_with_label ("Help");
   item2 = gtk_menu_item_new_with_label ("About");
   item3 = gtk_menu_item_new_with_label ("Preferences");
+
+#ifndef __arm__
   item4 = gtk_menu_item_new_with_label ("Quit");
+#endif
+
 
   g_signal_connect (G_OBJECT (item1), "activate",
                     G_CALLBACK (show_help_dialog), NULL);
@@ -1418,8 +1611,11 @@ int main (int argc, char **argv)
                     G_CALLBACK (show_about_dialog), NULL);
   g_signal_connect (G_OBJECT (item3), "activate",
                     G_CALLBACK (show_config_dialog), config_page);
+
+#ifndef __arm__
   g_signal_connect (G_OBJECT (item4), "activate",
                     G_CALLBACK (quit_msid), NULL);
+#endif
 
 #ifdef HILDON
   // use hildon menu stuff
@@ -1427,9 +1623,8 @@ int main (int argc, char **argv)
   gtk_menu_append (GTK_MENU (menu_bar), item1);
   gtk_menu_append (GTK_MENU (menu_bar), item2);
   gtk_menu_append (GTK_MENU (menu_bar), item3);
-  gtk_menu_append (GTK_MENU (menu_bar), item4);
-  hildon_program_set_common_menu (program, GTK_MENU (menu_bar));
 
+  hildon_program_set_common_menu (program, GTK_MENU (menu_bar));
 #else
   // use gtkmenubar
   menu_bar = gtk_menu_bar_new ();
@@ -1458,16 +1653,21 @@ int main (int argc, char **argv)
    */
 
 
-
   g_signal_connect (G_OBJECT (player_song_list), "key-release-event",
                     G_CALLBACK (key_on_selection), (gpointer) &machine);
 
   g_signal_connect (G_OBJECT (player_song_list), "row-activated",
-                    G_CALLBACK (play_selection), (gpointer) &machine);
-  g_signal_connect (G_OBJECT (player_song_list), "cursor-changed",
-                    G_CALLBACK (select_song), NULL);
+                    G_CALLBACK (handle_selection), (gpointer) &machine);
+
+  /*
+    g_signal_connect (G_OBJECT (player_song_list), "cursor-changed",
+    G_CALLBACK (select_song), NULL);
+  */
+
   g_signal_connect (G_OBJECT (player_play_button), "clicked",
                     G_CALLBACK (play_stop_song), (gpointer) &machine);
+  g_signal_connect (G_OBJECT (player_pause_button), "toggled",
+                    G_CALLBACK (pause_song), (gpointer) &machine);
   g_signal_connect (G_OBJECT (player_del_button), "clicked",
                     G_CALLBACK (delete_song), (gpointer) &machine);
 
@@ -1481,6 +1681,9 @@ int main (int argc, char **argv)
   g_signal_connect (G_OBJECT (hvsc_path_selection_button), "clicked",
 		    G_CALLBACK (select_hvsc_path), NULL);
 
+  g_signal_connect (G_OBJECT(output_mode_toggle), "toggled",
+		    G_CALLBACK (output_mode_changed), &machine);
+
   // FIXME - !!!
   g_signal_connect (G_OBJECT (image_button), "clicked",
 		    G_CALLBACK (load_screenshot), NULL);
